# fetchEx, AbortController и сетевая надежность

- [Тайм-аут при работе с HTTP-нодами GOLOS](./fetchEx.md#тайм-аут-при-работе-с-http-нодами-golos)
- [Использование AbortController в других сценариях](./fetchEx.md#использование-abortcontroller-в-других-сценариях)

#

Кроссплатформенный ponyfill для Fetch API и AbortController. 

Предназначен для использования в приложениях, созданных на основе GOLOS Blockchain.

Работает не только в браузерах, но и в **Node.js**, причем поддерживаются версии Node.js начиная от **11** (в отличие от стандартного AbortController, который поддерживает только Node.js 15+)

Позволяет легко отправлять HTTP-запросы с тайм-аутом на тот случай, если сервер зависнет из-за технической неполадки или еще что-то пойдет не так:

```js
import { fetchEx } from 'golos-lib-js/lib/utils' // или const { fetchEx } = golos.utils

let res = await fetchEx('https://dev.golos.app/api', {
    timeout: 5000,
    // ...и здесь любые опции fetch
})
res = await res.json()
```

В случае выхода за тайм-аут код не зависнет на длительный или неопределенный срок, а остановится и выбросит исключение `AbortError`. Которое можно обработать, выдать сообщение об ошибке пользователю, повторить запрос и так далее.  

Если не задавать опцию `timeout`, то будет использоваться значение по умолчанию. Оно лежит в `fetchEx.COMMON_TIMEOUT` и в данной версии библиотеки равно **10000 мсек**.

Можно также отключить `timeout`:
```js
let res = await fetchEx('https://dev.golos.app/api', {
    timeout: null
})
res = await res.json()
```

### Тайм-аут при работе с HTTP-нодами GOLOS

fetchEx также используется в самой библиотеке golos-lib-js для отправки API-запросов и транзакций в GOLOS, в том случае, если нода - https://, а не wss://.

В этом случае используется параметр `node_timeout`. По умолчанию это **30000 мсек**. 

Есть возможность задать свое значение, в зависимости от того, сколько ждать имеет смысл в вашем приложении и на какое качество интернет-подключения оно рассчитано.
Например, для десктопного приложения при отсутствии значительных объемов передаваемых данных достаточно и 5 сек:

```js
golos.config.set('node_timeout', 5000)
```

Параметр действует как для API-запросов, так и для отправки транзакций, в случае, если нода работает по HTTP. 

И поэтому мы не рекомендуем ставить значения менее 5-10 сек, ведь при медленном интернет-соединении они могут привести к ошибкам.

### Использование AbortController в других сценариях

Как уже было сказано, `fetchEx` имеет свою собственную реализацию `AbortController`, которая используется для тайм-аутов.

Но она доступна и для других вариантов использования. Например, чтобы прерывать запрос по нажатию кнопки Отмена пользователем:
```js
import { fetchEx } from 'golos-lib-js/lib/utils'
const { AbortController } = fetchEx

const ac = new AbortController()
let res = await fetchEx('https://dev.golos.app/api', {
    signal: ac.signal
})

// опция timeout в этом случае не работает, поэтому реализуем ее сами на основе setTimeout
setTimeout(() => ac.abort(), 10000)

document.getElementById('cancel').onclick = (e) => {
    ac.abort()
}
```
